from .copy import copy
from .corr import corr
from .min import min
from .max import max
from .use_only import use_only
from .rename import rename
from .max_diff import max_diff
from .time_intervals import time_intervals
from .resample import resample
from .slice_time import slice_time
from .info_metadata import info_metadata
from .info_vocabulary import info_vocabulary
from .drop import drop
from .reduce_memory import reduce_memory
